@@ -6,8 +6,8 @@
 #' @docType package
 #' @import stats utils compiler
 #' @importFrom R6 R6Class
-#' @importFrom grDevices dev.new dev.off pdf cairo_pdf rgb
-#' @importFrom graphics abline axis barplot boxplot hist lines matplot mtext par plot points strwidth text bxp grid
+#' @importFrom grDevices dev.new dev.off pdf
+#' @importFrom graphics abline axis barplot boxplot lines matplot mtext par plot points strwidth text bxp grid
 #'  
 #' 
 #' @details  License: GPL (>= 2)
@@ -59,25 +59,25 @@
 #'    sum(x * x - 10 * cos(2 * pi * x) + 10)
 #'  }
 #'  
-#'  ## We generate 200 instances (in this case, weights):
-#'  weights <- rnorm(200, mean = 0.9, sd = 0.02)
+#'  ## We generate 20 instances (in this case, weights):
+#'  weights <- rnorm(20, mean = 0.9, sd = 0.02)
 #'  
 #'  ## On this set of instances, we are interested in optimizing two
 #'  ## parameters of the SANN algorithm: tmax and temp. We setup the
 #'  ## parameter space as follows:
-#'  parameters.table <- '
-#'  tmax "" i (1, 5000)
+#'  parameters_table <- '
+#'  tmax "" i,log (1, 5000)
 #'  temp "" r (0, 100)
 #'  '
 #'  
 #'  ## We use the irace function readParameters to read this table:
-#'  parameters <- readParameters(text = parameters.table)
+#'  parameters <- readParameters(text = parameters_table)
 #'  
 #'  ## Next, we define the function that will evaluate each candidate
 #'  ## configuration on a single instance. For simplicity, we restrict to
 #'  ## three-dimensional functions and we set the maximum number of
-#'  ## iterations of SANN to 5000.
-#'  target.runner <- function(experiment, scenario)
+#'  ## iterations of SANN to 1000.
+#'  target_runner <- function(experiment, scenario)
 #'  {
 #'    instance <- experiment$instance
 #'    configuration <- experiment$configuration
@@ -89,7 +89,7 @@
 #'      return(weight * f_rastrigin(x) + (1 - weight) * f_rosenbrock(x))
 #'    }
 #'    res <- stats::optim(par,fn, method="SANN",
-#'                 control=list(maxit=5000
+#'                 control=list(maxit=1000
 #'                   , tmax = as.numeric(configuration[["tmax"]])
 #'                   , temp = as.numeric(configuration[["temp"]])
 #'                   ))
@@ -98,42 +98,42 @@
 #'    ## - 'error' is a string used to report an error
 #'    ## - 'outputRaw' is a string used to report the raw output of calls to
 #'    ##   an external program or function.
-#'    ## - 'call' is a string used to report how target.runner called the
+#'    ## - 'call' is a string used to report how target_runner called the
 #'    ##   external program or function.
 #'    return(list(cost = res$value))
 #'  }
 #'  
 #'  ## We define a configuration scenario by setting targetRunner to the
-#'  ## function define above, instances to the first 100 random weights, and
-#'  ## a maximum budget of 1000 calls to targetRunner.
-#'  scenario <- list(targetRunner = target.runner,
-#'                   instances = weights[1:100],
-#'                   maxExperiments = 1000,
+#'  ## function define above, instances to the first 10 random weights, and
+#'  ## a maximum budget of 'maxExperiments' calls to targetRunner.
+#'  scenario <- list(targetRunner = target_runner,
+#'                   instances = weights[1:10],
+#'                   maxExperiments = 500,
 #'                   # Do not create a logFile
 #'                   logFile = "")
 #'  
 #'  ## We check that the scenario is valid. This will also try to execute
-#'  ## target.runner.
+#'  ## target_runner.
 #'  checkIraceScenario(scenario, parameters = parameters)
 #'  
 #'  \donttest{
 #'  ## We are now ready to launch irace. We do it by means of the irace
 #'  ## function. The function will print information about its
 #'  ## progress. This may require a few minutes, so it is not run by default.
-#'  tuned.confs <- irace(scenario = scenario, parameters = parameters)
+#'  tuned_confs <- irace(scenario = scenario, parameters = parameters)
 #'  
 #'  ## We can print the best configurations found by irace as follows:
-#'  configurations.print(tuned.confs)
+#'  configurations.print(tuned_confs)
 #'  
 #'  ## We can evaluate the quality of the best configuration found by
 #'  ## irace versus the default configuration of the SANN algorithm on
-#'  ## the other 100 instances previously generated.
+#'  ## the other 10 instances previously generated.
 #'  ## To do so, first we apply the default configuration of the SANN
 #'  ## algorithm to these instances:
 #'  test <- function(configuration)
 #'  {
-#'    res <- lapply(weights[101:200],
-#'                  function(x) target.runner(
+#'    res <- lapply(weights[11:20],
+#'                  function(x) target_runner(
 #'                                experiment = list(instance = x,
 #'                                                  configuration = configuration),
 #'                                scenario = scenario))
@@ -143,7 +143,7 @@
 
 #'  ## We extract and apply the winning configuration found by irace
 #'  ## to these instances:
-#'  tuned <- test (removeConfigurationsMetaData(tuned.confs[1,]))
+#'  tuned <- test(removeConfigurationsMetaData(tuned_confs[1,]))
 #'  
 #'  ## Finally, we can compare using a boxplot the quality obtained with the
 #'  ## default parametrization of SANN and the quality obtained with the
@@ -156,3 +156,5 @@
 #' 
 NULL
 
+# Prefix for printing messages to the user.
+.msg.prefix <- "== irace == "
