@@ -25,6 +25,7 @@ recoverFromFile <- function(filename)
     # .Random.seed and .irace are special
     for (name in setdiff(names(iraceResults$state), c(".Random.seed", ".irace")))
       assign(name, iraceResults$state[[name]])
+    # FIXME: Check that irace.version matches and warn if not.
     assign(".Random.seed", iraceResults$state$.Random.seed, .GlobalEnv)
     for (name in ls(iraceResults$state$.irace))
       assign(name, get(name, envir = iraceResults$state$.irace), envir = .irace)
@@ -50,8 +51,8 @@ numeric.configurations.equal <- function(x, configurations, parameters, threshol
   selected <- 1:nrow(configurations)
   for (i in seq_along(param.names)) {
     param <- param.names[i]
-    lower <- parameters$domain[[param]][1]
-    upper <- parameters$domain[[param]][2]
+    x.domain <- getDependentBound(parameters, param, x)
+    x.range <- diff(x.domain)
  
     X <- x[[param]]
     # FIXME: Since at the end we select a subset of configurations, we could use selected here.
@@ -69,7 +70,20 @@ numeric.configurations.equal <- function(x, configurations, parameters, threshol
         # FIXME: Why is this updating d[j]? It seems that if the difference is
         # large for one configuration, then it will be assumed to be large for
         # the rest.
-        d[j] <- max(d[j], abs((as.numeric(X) - as.numeric(Y)) / (upper - lower)))
+        if (parameters$isDependent[param]) {
+          # Compare depedent domains by normalising their values to their own ranges first
+          # and calculating the difference. (When possible)
+          y.domain <- getDependentBound(parameters, param, configurations[selected[j],])
+          y.range <- diff(x.domain)
+          dx <- ifelse (x.range == 0, 0, (as.numeric(X) - x.domain[1]) / x.range)
+          dy <- ifelse (y.range == 0, 0, (as.numeric(Y) - y.domain[1]) / y.range)
+       
+          d[j] <- max(d[j], abs(dx - dy))
+        } else {
+          # FIXME: We should calculate (X - x.domain[1]) / x.range once for all configurations
+          # and all parameters, then calculate the differences using vectorization.
+          d[j] <- max(d[j], abs((as.numeric(X) - as.numeric(Y)) / x.range))
+        }
         if (d[j] > threshold) isSimilar.mat[j,i] <- FALSE
       }
     }
@@ -127,7 +141,7 @@ similarConfigurations <- function(configurations, parameters, threshold)
     ## filtering them out:
     configurations <- configurations [keepIdx, , drop=FALSE]
     ## filtering their strings out (to use them to define blocks):
-    strings <- strings [keepIdx]
+    strings <- strings[keepIdx]
     
     ## if everything is already filtered out, return
     if (nrow(configurations) == 0) {
@@ -191,10 +205,7 @@ similarConfigurations <- function(configurations, parameters, threshold)
 
 
 ## Number of iterations.
-computeNbIterations <- function(nbParameters)
-{
-  return (2 + log2(nbParameters))
-}
+computeNbIterations <- function(nbParameters) (2 + log2(nbParameters))
 
 ## Computational budget at each iteration.
 computeComputationalBudget <- function(remainingBudget, indexIteration,
@@ -219,10 +230,7 @@ computeNbConfigurations <- function(currentBudget, indexIteration, firstTest, ea
 
 ## Termination of a race at each iteration. The race will stop if the
 ## number of surviving configurations is equal or less than this number.
-computeTerminationOfRace <- function(nbParameters)
-{
-  return (2 + log2(nbParameters))
-}
+computeTerminationOfRace <- function(nbParameters) (2 + log2(nbParameters))
 
 ## Compute the minimum budget required, and exit early in case the
 ## budget given by the user is insufficient.
@@ -330,10 +338,15 @@ startParallel <- function(scenario)
     } else {
       requireNamespace("parallel", quietly = TRUE)
       if (.Platform$OS.type == 'windows' && is.null(.irace$cluster)) {
+        # FIXME: makeCluster does not print the output generated by the workers
+        # on Windows. We need to use the future package for that:
+        # https://stackoverflow.com/questions/56501937/how-to-print-from-clusterapply
         .irace$cluster <- parallel::makeCluster(parallel)
+        if (scenario$debugLevel >= 1) irace.note("makeCluster initialized for ", parallel, " jobs.")
         # In Windows, this needs to be exported, or we get:
         ## Error in checkForRemoteErrors(val) : 
         ##  2 nodes produced errors; first error: could not find function "target.runner"
+        parallel::clusterExport(.irace$cluster, ls(environment(startParallel)), envir=environment(startParallel))
         parallel::clusterExport(.irace$cluster, list("target.runner"), envir=.irace)
         # In addition, we export the global environment because the user may
         # have defined stuff there. There must be a better way to do this, but
@@ -347,7 +360,7 @@ startParallel <- function(scenario)
 stopParallel <- function()
 {
   if (!is.null(.irace$cluster)) {
-    parallel::stopCluster(.irace$cluster)
+    try(parallel::stopCluster(.irace$cluster), silent=TRUE)
     .irace$cluster <- NULL
   }
 }
@@ -356,6 +369,7 @@ irace.init <- function(scenario)
 {
   # We need to do this here to use/recover .Random.seed later.
   if (is.na(scenario$seed)) {
+    # FIXME: We should store this seed in state not in scenario. We should not modify scenario.
     scenario$seed <- trunc(runif(1, 1, .Machine$integer.max))
   }
   set.seed(scenario$seed)
@@ -363,7 +377,7 @@ irace.init <- function(scenario)
     irace.note("RNGkind: ", paste0(RNGkind(), collapse = " "), "\n")
     irace.note(".Random.seed: ", paste0(.Random.seed, collapse = ", "), "\n")
   }
-  return(scenario)
+  scenario
 }
 
 ## Generate instances + seed.
@@ -380,14 +394,13 @@ generateInstances <- function(scenario, remainingBudget)
     # Sample instances index in groups (ntimes)
     sindex <- as.vector(sapply(rep(length(instances), ntimes), sample.int, replace = FALSE))
   } else {
-    sindex <- rep(1:length(instances), ntimes)
+    sindex <- rep(1L:length(instances), ntimes)
   }
   # Sample seeds.
   # 2147483647 is the maximum value for a 32-bit signed integer.
   # We use replace = TRUE, because replace = FALSE allocates memory for each possible number.
-  tmp <- data.frame (instance = sindex,
-                     seed = sample.int(2147483647, size = ntimes * length(instances), replace = TRUE))
-  return(tmp)
+  data.frame(instance = sindex,
+             seed = sample.int(2147483647L, size = length(sindex), replace = TRUE), stringsAsFactors=FALSE)
 }
 
 addInstances <- function(scenario, instancesList, n.instances)
@@ -396,7 +409,7 @@ addInstances <- function(scenario, instancesList, n.instances)
   if (is.null.or.empty(instancesList))
     instancesList <- generateInstances(scenario, n.instances)
   # If deterministic, we have already added all instances.
-  else if (! scenario$deterministic)
+  else if (!scenario$deterministic)
     instancesList <- rbind(instancesList, generateInstances(scenario, n.instances))
 
   # FIXME: Something is adding rownames. Clear them to avoid future problems.
@@ -469,7 +482,7 @@ allConfigurationsInit <- function(scenario, parameters)
                     scenario$configurationsFile, "'.")
     cat("# Adding", nrow(initConfigurations), "initial configuration(s)\n")
     if (scenario$debugLevel >= 2)
-      print(as.data.frame(scenario$initConfigurations, stringAsFactor = FALSE), digits=15)
+      print(as.data.frame(scenario$initConfigurations, stringsAsFactors = FALSE), digits=15)
   } else {
     initConfigurations <- confs_from_file
   }
@@ -491,49 +504,102 @@ allConfigurationsInit <- function(scenario, parameters)
     allConfigurations <-
       as.data.frame(matrix(ncol = length(configurations.colnames),
                            nrow = 0,
-                           dimnames = list(NULL, configurations.colnames)))
+                           dimnames = list(NULL, configurations.colnames)),
+                    stringsAsFactors=FALSE)
   }
-  return(allConfigurations)
+  allConfigurations
 }
 
 #' irace
 #'
-#' \code{irace} implements iterated Race. It receives some parameters to be tuned 
+#' `irace` implements iterated Race. It receives some parameters to be tuned 
 #'   and returns the best configurations found, namely, the elite configurations 
 #'   obtained from the last iterations (and sorted by rank).
 #' 
 #' @template arg_scenario
 #' @template arg_parameters
 #'
-#' @details The function \code{irace} executes the tuning procedure using 
-#'  the information provided in \code{scenario} and \code{parameters}. Initially it checks 
-#'  the correctness of \code{scenario} and recovers a previous execution if 
-#'  \code{scenario$recoveryFile} is set. A R data file log of the execution is created 
-#'  in \code{scenario$logFile}.
+#' @details The function `irace` executes the tuning procedure using 
+#'  the information provided in `scenario` and `parameters`. Initially it checks 
+#'  the correctness of `scenario` and recovers a previous execution if 
+#'  `scenario$recoveryFile` is set. A R data file log of the execution is created 
+#'  in `scenario$logFile`.
 #'
 #' @template return_irace
 #' @examples
 #' \dontrun{
 #' parameters <- readParameters("parameters.txt")
-#' scenario <- readScenario(filename = "scenario.txt",
-#'                          scenario = defaultScenario())
+#' scenario <- readScenario(filename = "scenario.txt")
 #' irace(scenario = scenario, parameters = parameters)
 #' }
 #'
 #' @seealso
 #'  \describe{
-#'  \item{\code{\link{irace.main}}}{a higher-level command-line interface to \code{irace}.}
-#'  \item{\code{\link{readScenario}}}{for reading a configuration scenario from a file.}
-#'  \item{\code{\link{readParameters}}}{read the target algorithm parameters from a file.}
-#'  \item{\code{\link{defaultScenario}}}{returns the default scenario settings of \pkg{irace}.}
-#'  \item{\code{\link{checkScenario}}}{to check that the scenario is valid.}
+#'  \item{[irace.main()]}{a higher-level interface to `irace`.}
+#'  \item{[irace.cmdline()]}{a command-line interface to `irace`.}
+#'  \item{[readScenario()]}{for reading a configuration scenario from a file.}
+#'  \item{[readParameters()]}{read the target algorithm parameters from a file.}
+#'  \item{[defaultScenario()]}{returns the default scenario settings of \pkg{irace}.}
+#'  \item{[checkScenario()]}{to check that the scenario is valid.}
 #' }
 #' 
 #' @author Manuel López-Ibáñez and Jérémie Dubois-Lacoste
+#' @concept running
 #' @export
 irace <- function(scenario, parameters)
+  irace_common(scenario, parameters, simple = TRUE)
+
+irace_common <- function(scenario, parameters, simple, output.width = 9999L)
 {
-  catInfo <- function(..., verbose = TRUE) {
+  if (!simple) {
+    op <- options(width = output.width) # Do not wrap the output.
+    on.exit(options(op), add = TRUE)
+  }
+  scenario <- checkScenario(scenario)
+  debugLevel <- scenario$debugLevel
+
+  if (debugLevel >= 1) {
+    op.debug <- options(warning.length = 8170,
+                        error = if (interactive()) utils::recover
+                                else irace.dump.frames)
+    on.exit(options(op.debug), add = TRUE)
+    printScenario (scenario)
+  }
+  
+  if (missing(parameters)) {
+    # Read parameters definition
+    parameters <- readParameters (file = scenario$parameterFile,
+                                  digits = scenario$digits,
+                                  debugLevel = debugLevel)
+  } else {
+    parameters <- checkParameters(parameters)
+  }
+
+  eliteConfigurations <- irace_run(scenario = scenario, parameters = parameters)
+  if (simple) return(eliteConfigurations)
+
+  if (!scenario$quiet) {
+    cat("# Best configurations (first number is the configuration ID;",
+        " listed from best to worst according to the ",
+        test.type.order.str(scenario$testType), "):\n", sep = "")
+    configurations.print(eliteConfigurations)
+  
+    cat("# Best configurations as commandlines (first number is the configuration ID; same order as above):\n")
+    configurations.print.command (eliteConfigurations, parameters)
+  }
+  
+  if (scenario$postselection > 0) 
+    psRace(iraceLogFile=scenario$logFile, postselection=scenario$postselection, elites=TRUE)
+  
+  testing_fromlog(logFile = scenario$logFile)
+  
+  invisible(eliteConfigurations)
+}
+
+irace_run <- function(scenario, parameters)
+{
+  quiet <- scenario$quiet
+  catInfo <- if (quiet) do_nothing else function(..., verbose = TRUE) {
     irace.note (..., "\n")
     if (verbose) {
       cat ("# Iteration: ", indexIteration, "\n",
@@ -541,36 +607,47 @@ irace <- function(scenario, parameters)
            "# experimentsUsedSoFar: ", experimentsUsedSoFar, "\n",
            "# timeUsed: ", timeUsed, "\n",
            "# remainingBudget: ", remainingBudget, "\n",
-           "# currentBudget: ", currentBudget, "\n",
+             "# currentBudget: ", currentBudget, "\n",
            "# number of elites: ", nrow(eliteConfigurations), "\n",
            "# nbConfigurations: ", nbConfigurations, "\n",
            sep = "")
     }
   }
-  
-  scenario <- checkScenario(defaultScenario(scenario))
+
+  irace_finish <- function(iraceResults, scenario, reason) {
+    elapsed <- timer$elapsed()
+    if (!quiet)
+      cat("# Total CPU user time: ", elapsed["user"], ", CPU sys time: ", elapsed["system"],
+          ", Wall-clock time: ", elapsed["wallclock"], "\n", sep="")
+    iraceResults$state$elapsed = elapsed
+    iraceResults$state$completed = reason
+    irace_save_logfile(iraceResults, scenario)
+    iraceResults$state$eliteConfigurations
+  }
+
+  timer <- Timer$new()
+  debugLevel <- scenario$debugLevel
 
   # Recover state from file?
-  if (!is.null(scenario$recoveryFile)) {
+  if (!is.null.or.empty(scenario$recoveryFile)) {
     irace.note ("Resuming from file: '", scenario$recoveryFile,"'\n")
     recoverFromFile(scenario$recoveryFile)
     # We call checkScenario again to fix any inconsistencies in the recovered
     # data.
-    firstRace <- FALSE
+    # FIXME: Do not call checkScenario earlier and instead do the minimum to check recoveryFile.
     scenario <- checkScenario(scenario)
+    firstRace <- FALSE
+    stopParallel()
     startParallel(scenario)
     on.exit(stopParallel(), add = TRUE)
-
   } else { # Do not recover
     firstRace <- TRUE
-    scenario <- irace.init (scenario)
+    scenario <- irace.init(scenario)
     forbiddenExps <- scenario$forbiddenExps
-    debugLevel <- scenario$debugLevel
     # Set options controlling debug level.
     # FIXME: This should be the other way around, the options set the debugLevel.
     options(.race.debug.level = debugLevel)
     options(.irace.debug.level = debugLevel)
-    
     # Create a data frame of all configurations ever generated.
     allConfigurations <- allConfigurationsInit(scenario, parameters)
     nbUserConfigurations <- nrow(allConfigurations)
@@ -588,7 +665,7 @@ irace <- function(scenario, parameters)
     )
     model <- NULL
     nbConfigurations <- 0
-    eliteConfigurations <- data.frame()
+    eliteConfigurations <- data.frame(stringsAsFactors=FALSE)
     
     nbIterations <- ifelse (scenario$nbIterations == 0,
                             computeNbIterations(parameters$nbVariable),
@@ -652,8 +729,8 @@ irace <- function(scenario, parameters)
         output <- do.experiments(configurations = allConfigurations[next.configuration:nconfigurations, ],
                                  ninstances = ninstances, scenario = scenario, parameters = parameters)  
         iraceResults$experimentLog <- rbind(iraceResults$experimentLog,
-                                            cbind(rep(0, nrow(output$experimentLog)),
-                                                  output$experimentLog)) 
+                                            # These experiments are assigned iteration 0
+                                            cbind(iteration=0L, output$experimentLog)) 
         
         iraceResults$experiments <- merge.matrix (iraceResults$experiments,
                                                   output$experiments)
@@ -687,7 +764,7 @@ irace <- function(scenario, parameters)
         } else {
           nconfigurations <- min(1024, nconfigurations + new.conf)
         }
-      }
+      } # end of while(TRUE)
       
       if (length(rejectedIDs) > 0) {
         irace.note ("Immediately rejected configurations: ",
@@ -764,11 +841,11 @@ irace <- function(scenario, parameters)
           if (scenario$capping) 
             paste0("# capping: ", scenario$cappingType, "\n",
                    "# type bound: ", scenario$boundType, "\n", 
-                   "# maxBound: ", scenario$boundMax, "\n",
+                   "# boundMax: ", scenario$boundMax, "\n",
                    "# par bound: ", scenario$boundPar, "\n", 
                    "# bound digits: ", scenario$boundDigits, "\n")
           else if (!is.null(scenario$boundMax))
-            paste0("# maxBound: ", scenario$boundMax, "\n"),
+            paste0("# boundMax: ", scenario$boundMax, "\n"),
           verbose = FALSE)
 
   
@@ -790,28 +867,25 @@ irace <- function(scenario, parameters)
                                boundEstimate = boundEstimate,
                                rejectedIDs = rejectedIDs,
                                forbiddenExps = forbiddenExps,
-                               completed = list(flag=FALSE, msg=""))
-    # Consistency checks
-    irace.assert(sum(!is.na(iraceResults$experiments)) == experimentsUsedSoFar)
-    irace.assert(nrow(iraceResults$experimentLog) == experimentsUsedSoFar)
-
+                               completed = "Incomplete")
     ## Save to the log file
     iraceResults$allConfigurations <- allConfigurations
     irace_save_logfile(iraceResults, scenario)
 
+    # Consistency checks
+    irace.assert(nrow(iraceResults$experimentLog) == experimentsUsedSoFar)
+    # With elitist=0 we may re-run the same configuration on the same (instance,seed) pair
+    # FIXME: This assert is failing when sampleInstances=FALSE because we are re-executing again an instance that was executed at the start of the race. 
+    if (FALSE && scenario$elitist)
+      irace.assert(sum(!is.na(iraceResults$experiments)) == experimentsUsedSoFar)
+
     if (remainingBudget <= 0) {
       catInfo("Stopped because budget is exhausted")
-      iraceResults$state$completed$flag = TRUE
-      iraceResults$state$completed$msg = "Budget exhausted"
-      irace_save_logfile(iraceResults, scenario)
-      return (eliteConfigurations)
+      return(irace_finish(iraceResults, scenario, reason = "Budget exhausted"))
     }
     if (scenario$maxTime > 0 && timeUsed >= scenario$maxTime) {
       catInfo("Stopped because time budget is exhausted")
-      iraceResults$state$completed$flag = TRUE
-      iraceResults$state$completed$msg = "Time budget exhausted"
-      irace_save_logfile(iraceResults, scenario)
-      return (eliteConfigurations)
+      return(irace_finish(iraceResults, scenario, reason = "Time budget exhausted"))
     }
 
     if (indexIteration > nbIterations) {
@@ -821,10 +895,7 @@ irace <- function(scenario, parameters)
         if (debugLevel >= 1) {
           catInfo("Limit of iterations reached", verbose = FALSE)
         }
-        iraceResults$state$completed$flag = TRUE
-        iraceResults$state$completed$msg = "Limit of iterations reached"
-        irace_save_logfile(iraceResults, scenario)
-        return (eliteConfigurations)
+        return(irace_finish(iraceResults, scenario, reason = "Limit of iterations reached"))
       }
     }
     # Compute the current budget (nb of experiments for this iteration),
@@ -868,10 +939,7 @@ irace <- function(scenario, parameters)
       } else {
         catInfo("Stopped because ",
                 "there is not enough budget to enforce the value of nbConfigurations.")
-        iraceResults$state$completed$flag = TRUE
-        iraceResults$state$completed$msg = "Not enough budget to enforce the value of nbConfigurations"
-        irace_save_logfile(iraceResults, scenario)
-        return (eliteConfigurations)
+        return(irace_finish(iraceResults, scenario, reason = "Not enough budget to enforce the value of nbConfigurations"))
       }
     }
     
@@ -880,10 +948,7 @@ irace <- function(scenario, parameters)
       catInfo("Stopped because there is not enough budget left to race more than ",
               "the minimum (", minSurvival,")\n",
               "# You may either increase the budget or set 'minNbSurvival' to a lower value")
-      iraceResults$state$completed$flag = TRUE
-      iraceResults$state$completed$msg = "Not enough budget to race more than the minimum configurations"
-      irace_save_logfile(iraceResults, scenario)
-      return (eliteConfigurations)
+      return(irace_finish(iraceResults, scenario, reason = "Not enough budget to race more than the minimum configurations"))
     }
 
 
@@ -901,10 +966,7 @@ irace <- function(scenario, parameters)
       catInfo("Stopped because ",
               "there is not enough budget left to race newly sampled configurations")
       #(number of elites  + 1) * (mu + min(5, indexIteration)) > remainingBudget" 
-      iraceResults$state$completed$flag = TRUE
-      iraceResults$state$completed$msg = "Not enough budget left to race newly sampled configurations"
-      irace_save_logfile(iraceResults, scenario)
-      return (eliteConfigurations)
+      return(irace_finish(iraceResults, scenario, reason = "Not enough budget left to race newly sampled configurations"))
     }
     
     if (scenario$elitist) {
@@ -914,18 +976,12 @@ irace <- function(scenario, parameters)
           + nrow(eliteConfigurations) * min(scenario$elitistNewInstances, max(scenario$mu, scenario$firstTest))
           > currentBudget) {
         catInfo("Stopped because there is not enough budget left to race all configurations up to the first test (or mu)")
-        iraceResults$state$completed$flag = TRUE
-        iraceResults$state$completed$msg = "Not enough budget to race all configurations up to the first test (or mu)"
-        irace_save_logfile(iraceResults, scenario)
-        return (eliteConfigurations)
+        return(irace_finish(iraceResults, scenario, reason = "Not enough budget to race all configurations up to the first test (or mu)"))
       }
     } else if (nbConfigurations * max(scenario$mu, scenario$firstTest)
                > currentBudget) {
       catInfo("Stopped because there is not enough budget left to race all configurations up to the first test (or mu)")
-      iraceResults$state$completed$flag = TRUE
-      iraceResults$state$completed$msg  = "Not enough budget to race all configurations up to the first test (or mu)"
-      irace_save_logfile(iraceResults, scenario)
-      return (eliteConfigurations)
+      return(irace_finish(iraceResults, scenario, reason = "Not enough budget to race all configurations up to the first test (or mu)"))
     }
 
     catInfo("Iteration ", indexIteration, " of ", nbIterations, "\n",
@@ -959,7 +1015,7 @@ irace <- function(scenario, parameters)
         allConfigurations <- rbind(allConfigurations, newConfigurations)
         rownames(allConfigurations) <- allConfigurations$.ID.
         raceConfigurations <- allConfigurations[allConfigurations$.ID. %!in% rejectedIDs, , drop = FALSE]
-      } else if (nbNewConfigurations < 0) {
+      } else if (nbNewConfigurations <= 0) {
         # We let the user know that not all configurations will be used.
         if (nbUserConfigurations > nbConfigurations) {
           catInfo("Only ", nbConfigurations,
@@ -982,14 +1038,13 @@ irace <- function(scenario, parameters)
       nbNewConfigurations <- nbConfigurations - nrow(eliteConfigurations)
 
       # Update the model based on elites configurations
-      if (debugLevel >= 1) { irace.note("Update model\n") }
+      if (debugLevel >= 1) irace.note("Update model\n")
       model <- updateModel(parameters, eliteConfigurations, model, indexIteration,
                            nbIterations, nbNewConfigurations, scenario)
-      if (debugLevel >= 2) { printModel (model) }
-      
-      if (debugLevel >= 1) {
+      if (debugLevel >= 2) printModel (model)
+      if (debugLevel >= 1)
         irace.note("Sample ", nbNewConfigurations, " configurations from model\n")
-      }
+      
       newConfigurations <- sampleModel(parameters, eliteConfigurations,
                                        model, nbNewConfigurations,
                                        digits = scenario$digits,
@@ -1116,12 +1171,13 @@ irace <- function(scenario, parameters)
     # FIXME: Since we only actually keep the alive ones, we don't need
     # to carry around rejected ones in raceResults$configurations. This
     # would reduce overhead.
-    eliteConfigurations <- extractElites(raceResults$configurations,
+    eliteConfigurations <- extractElites(scenario, parameters,
+                                         raceResults$configurations,
                                          min(raceResults$nbAlive, minSurvival))
     irace.note("Elite configurations (first number is the configuration ID;",
                " listed from best to worst according to the ",
                test.type.order.str(scenario$testType), "):\n")
-    configurations.print(eliteConfigurations, metadata = debugLevel >= 1)
+    if (!quiet) configurations.print(eliteConfigurations, metadata = debugLevel >= 1)
     iraceResults$iterationElites <- c(iraceResults$iterationElites, eliteConfigurations$.ID.[1])
     iraceResults$allElites[[indexIteration]] <- eliteConfigurations$.ID.
     
@@ -1146,9 +1202,5 @@ irace <- function(scenario, parameters)
       irace.print.memUsed()
     }
   }
-  # This code is actually never executed because we return above.
-  # Leslie: adding this just in case 
-  iraceResults$state$completed$flag = TRUE
-  irace_save_logfile(iraceResults, scenario)
-  return (eliteConfigurations)
+  irace.internal.error("This code is actually never executed because we return above")
 }
